@@ -3,7 +3,7 @@ import numpy as np
 from matplotlib import pyplot as plt
 
 
-# custom visualization, x and y are taken from outer scope
+# custom visualization; x, y, model are taken from outer scope
 def viz(args):
     plt.plot(x, y, "ok")
     xm = np.linspace(x[0], x[-1], 100)
@@ -18,4 +18,5 @@ x = np.array([1, 2, 3, 4, 5])
 y = np.array([1.03, 1.58, 2.03, 2.37, 3.09])
 c = cost.LeastSquares(x, y, 0.1, model)
 m = Minuit(c, 0.5, 0.5)
-m.interactive(viz)  # calling m.interactive() uses LeastSquares.visualize
+m.interactive(viz)
+# m.interactive() also works and calls LeastSquares.visualize
