@@ -7,12 +7,22 @@ project_dir = PurePath(__file__).parent.parent
 git_submodule = subp.check_output(
     ["git", "submodule", "status"], cwd=project_dir
 ).decode()
+
 for item in git_submodule.strip().split("\n"):
     parts = item.split()
     if PurePath(parts[1]) != PurePath("extern") / "root":
         continue
 
     assert len(parts) == 3, "module is not checked out"
-
-    root_version = parts[2][1:-1]  # strip braces
     break
+
+# git submodule status does not yield the right state
+# we must use git describe --tags
+root_version = (
+    subp.check_output(
+        ["git", "describe", "--tags"], cwd=project_dir / "extern" / "root"
+    )
+    .decode()
+    .strip()
+)
+print(root_version)
