@@ -11,6 +11,7 @@
 #define ROOT_Minuit2_AnalyticalGradientCalculator
 
 #include "Minuit2/GradientCalculator.h"
+#include "Minuit2/MnMatrixfwd.h"
 
 namespace ROOT {
 
@@ -19,11 +20,12 @@ namespace Minuit2 {
 class FCNGradientBase;
 class MnUserTransformation;
 
+
 class AnalyticalGradientCalculator : public GradientCalculator {
 
 public:
    AnalyticalGradientCalculator(const FCNGradientBase &fcn, const MnUserTransformation &state)
-      : fGradCalc(fcn), fTransformation(state)
+      : fGradFunc(fcn), fTransformation(state)
    {
    }
 
@@ -33,10 +35,20 @@ public:
 
    FunctionGradient operator()(const MinimumParameters &, const FunctionGradient &) const override;
 
+   /// compute Hessian matrix
+   bool Hessian(const MinimumParameters &, MnAlgebraicSymMatrix &) const override;
+
+   /// compute second derivatives (diagonal of Hessian)
+   bool G2(const MinimumParameters &, MnAlgebraicVector &) const override;
+
    virtual bool CheckGradient() const;
 
+   virtual bool CanComputeG2() const;
+
+   virtual bool CanComputeHessian() const;
+
 protected:
-   const FCNGradientBase &fGradCalc;
+   const FCNGradientBase &fGradFunc;
    const MnUserTransformation &fTransformation;
 };
 
