@@ -14,6 +14,7 @@
 #include "Minuit2/MnPrint.h"
 
 #include <vector>
+#include <functional>
 
 namespace ROOT {
 
@@ -46,24 +47,12 @@ public:
    std::vector<double> Gradient(const std::vector<double> &v) const override
    {
       fFunc.Gradient(&v[0], &fGrad[0]);
-
-      MnPrint("FCNGradAdapter").Debug([&](std::ostream &os) {
-         os << "gradient in FCNAdapter = {";
-         for (unsigned int i = 0; i < fGrad.size(); ++i)
-            os << fGrad[i] << (i == fGrad.size() - 1 ? '}' : '\t');
-      });
       return fGrad;
    }
    std::vector<double> GradientWithPrevResult(const std::vector<double> &v, double *previous_grad, double *previous_g2,
                                               double *previous_gstep) const override
    {
       fFunc.GradientWithPrevResult(&v[0], &fGrad[0], previous_grad, previous_g2, previous_gstep);
-
-      MnPrint("FCNGradAdapter").Debug([&](std::ostream &os) {
-         os << "gradient in FCNAdapter = {";
-         for (unsigned int i = 0; i < fGrad.size(); ++i)
-            os << fGrad[i] << (i == fGrad.size() - 1 ? '}' : '\t');
-      });
       return fGrad;
    }
    // forward interface
@@ -78,10 +67,68 @@ public:
       }
    }
 
+   /// return second derivatives (diagonal of the Hessian matrix)
+   std::vector<double> G2(const std::vector<double> & x) const override {
+      if (fG2Func)
+         return fG2Func(x);
+      if (fHessianFunc) {
+         unsigned int n = fFunc.NDim();
+         if (fG2Vec.empty() ) fG2Vec.resize(n);
+         if (fHessian.empty() ) fHessian.resize(n*n);
+         fHessianFunc(x,fHessian.data());
+         if (!fHessian.empty()) {
+            // get diagonal element of h
+            for (unsigned int i = 0; i < n; i++)
+               fG2Vec[i] = fHessian[i*n+i];
+         }
+         else fG2Vec.clear();
+      }
+      else
+         if (!fG2Vec.empty()) fG2Vec.clear();
+      return fG2Vec;
+   }
+
+   /// compute Hessian. Return Hessian as a std::vector of size(n*n)
+   std::vector<double> Hessian(const std::vector<double> & x ) const override {
+      unsigned int n = fFunc.NDim();
+      if (fHessianFunc) {
+         if (fHessian.empty() ) fHessian.resize(n * n);
+         bool ret = fHessianFunc(x,fHessian.data());
+         if (!ret) {
+            fHessian.clear();
+            fHessianFunc = nullptr;
+         }
+      } else {
+         fHessian.clear();
+      }
+
+      return fHessian;
+   }
+
+   bool HasG2() const override {
+      return bool(fG2Func);
+   }
+   bool HasHessian() const override {
+      return bool(fHessianFunc);
+   }
+
+   template<class Func>
+   void SetG2Function(Func f) { fG2Func = f;}
+
+   template<class Func>
+   void SetHessianFunction(Func f) { fHessianFunc = f;}
+
+   void SetErrorDef(double up) override { fUp = up; }
+
 private:
    const Function &fFunc;
    double fUp;
    mutable std::vector<double> fGrad;
+   mutable std::vector<double> fHessian;
+   mutable std::vector<double> fG2Vec;
+
+   std::function<std::vector<double>(const std::vector<double> &)> fG2Func;
+   mutable std::function<bool(const std::vector<double> &, double *)> fHessianFunc;
 };
 
 } // end namespace Minuit2
