@@ -10,7 +10,7 @@
 #ifndef ROOT_Minuit2_FumiliGradientCalculator
 #define ROOT_Minuit2_FumiliGradientCalculator
 
-#include "Minuit2/GradientCalculator.h"
+#include "Minuit2/AnalyticalGradientCalculator.h"
 #include "Minuit2/MnMatrix.h"
 
 namespace ROOT {
@@ -20,13 +20,12 @@ namespace Minuit2 {
 class FumiliFCNBase;
 class MnUserTransformation;
 
-class FumiliGradientCalculator : public GradientCalculator {
+/// Fumili gradient calculator using external gradient provided by FCN
+/// Note that the computed Hessian and G2 are an approximation valid for small residuals
+class FumiliGradientCalculator : public AnalyticalGradientCalculator {
 
 public:
-   FumiliGradientCalculator(const FumiliFCNBase &fcn, const MnUserTransformation &state, int n)
-      : fFcn(fcn), fTransformation(state), fHessian(MnAlgebraicSymMatrix(n))
-   {
-   }
+   FumiliGradientCalculator(const FumiliFCNBase &fcn, const MnUserTransformation &trafo, int n);
 
    ~FumiliGradientCalculator() override {}
 
@@ -36,11 +35,19 @@ public:
 
    const MnUserTransformation &Trafo() const { return fTransformation; }
 
-   const MnAlgebraicSymMatrix &Hessian() const { return fHessian; }
+   const MnAlgebraicSymMatrix &GetHessian() const { return fHessian; }
+
+   bool Hessian(const MinimumParameters &, MnAlgebraicSymMatrix &) const override;
+
+   bool G2(const MinimumParameters &, MnAlgebraicVector &) const override;
+
+   bool CanComputeG2() const override { return true;}
+
+   bool CanComputeHessian() const override { return true;}
+
 
 private:
    const FumiliFCNBase &fFcn;
-   const MnUserTransformation &fTransformation;
    mutable MnAlgebraicSymMatrix fHessian;
 };
 
