@@ -21,6 +21,7 @@
 
 #include <vector>
 #include <string>
+#include <functional>
 
 namespace ROOT {
 
@@ -45,7 +46,7 @@ namespace Minuit2 {
    Using a string  (used by the plugin manager) or via an enumeration
    an one can set all the possible minimization algorithms (Migrad, Simplex, Combined, Scan and Fumili).
 
-   Refer to the [guide](https://root.cern.ch/root/htmldoc/guides/minuit2/Minuit2.html) for an introduction how Minuit
+   Refer to the [guide](https://root.cern/root/htmldoc/guides/minuit2/Minuit2.html) for an introduction how Minuit
    works.
 
    @ingroup Minuit
@@ -88,8 +89,8 @@ public:
    /// set the function to minimize
    void SetFunction(const ROOT::Math::IMultiGenFunction &func) override;
 
-   /// set gradient the function to minimize
-   void SetFunction(const ROOT::Math::IMultiGradFunction &func) override;
+   /// set the function implementing Hessian computation
+   void SetHessianFunction(std::function<bool(const std::vector<double> &, double *)> hfunc) override;
 
    /// set free variable
    bool SetVariable(unsigned int ivar, const std::string &name, double val, double step) override;
@@ -156,7 +157,7 @@ public:
    const double *X() const override { return &fValues.front(); }
 
    /// return pointer to gradient values at the minimum
-   const double *MinGradient() const override { return 0; } // not available in Minuit2
+   const double *MinGradient() const override { return nullptr; } // not available in Minuit2
 
    /// number of function calls to reach the minimum
    unsigned int NCalls() const override { return fState.NFcn(); }
