@@ -15,20 +15,13 @@
 
 #include "Minuit2/MnConfig.h"
 
+// define typedf's in MnMatrixfwd
+#include "Minuit2/MnMatrixfwd.h"
+
 #include "Minuit2/LASymMatrix.h"
 #include "Minuit2/LAVector.h"
 #include "Minuit2/LaInverse.h"
 #include "Minuit2/LaOuterProduct.h"
 
-namespace ROOT {
-
-namespace Minuit2 {
-
-typedef LASymMatrix MnAlgebraicSymMatrix;
-typedef LAVector MnAlgebraicVector;
-
-} // namespace Minuit2
-
-} // namespace ROOT
 
 #endif // ROOT_Minuit2_MnMatrix
