@@ -25,13 +25,13 @@ class MnSeedGenerator : public MinimumSeedGenerator {
 public:
    MnSeedGenerator() {}
 
-   virtual ~MnSeedGenerator() {}
+   ~MnSeedGenerator() override {}
 
-   virtual MinimumSeed
-   operator()(const MnFcn &, const GradientCalculator &, const MnUserParameterState &, const MnStrategy &) const;
+   MinimumSeed
+   operator()(const MnFcn &, const GradientCalculator &, const MnUserParameterState &, const MnStrategy &) const override;
 
-   virtual MinimumSeed operator()(const MnFcn &, const AnalyticalGradientCalculator &, const MnUserParameterState &,
-                                  const MnStrategy &) const;
+   MinimumSeed operator()(const MnFcn &, const AnalyticalGradientCalculator &, const MnUserParameterState &,
+                                  const MnStrategy &) const override;
 
 private:
 };
