@@ -1,6 +1,6 @@
+#include "pybind11.hpp"
 #include <Minuit2/FCNGradientBase.h>
 #include <cstdint>
-#include <pybind11/pytypes.h>
 #include <vector>
 
 namespace py = pybind11;
