@@ -1,7 +1,10 @@
 """Minuit class."""
 
+from __future__ import annotations
+
 import warnings
 from iminuit import util as mutil
+from iminuit.util import _replace_none as replace_none
 from iminuit._core import (
     FCN,
     MnContours,
@@ -15,10 +18,10 @@ from iminuit._core import (
     MnUserParameterState,
     FunctionMinimum,
 )
+from iminuit.warnings import ErrordefAlreadySetWarning
 import numpy as np
 from typing import (
     Union,
-    Sequence,
     Optional,
     Callable,
     Tuple,
@@ -30,12 +33,9 @@ from typing import (
     Set,
     Sized,
 )
-from iminuit.typing import UserBound
+from iminuit.typing import UserBound, Cost, CostGradient
 from iminuit._optional_dependencies import optional_module_for
-
-# Better use numpy.typing.ArrayLike in the future, but this
-# requires dropping Python-3.6 support
-_ArrayLike = Sequence
+from numpy.typing import ArrayLike
 
 MnPrint.global_level = 0
 
@@ -111,6 +111,13 @@ class Minuit:
         the :download:`MINUIT2 User's Guide <mnusersguide.pdf>`. This parameter is also
         called *UP* in MINUIT documents.
 
+        If FCN has an attribute ``errordef``, its value is used automatically and you
+        should not set errordef by hand. Doing so will raise a
+        ErrordefAlreadySetWarning.
+
+        For the builtin cost functions in :mod:`iminuit.cost`, you don't need to set
+        this value, because they all have the ``errordef`` attribute set.
+
         To make user code more readable, we provided two named constants::
 
             m_lsq = Minuit(a_least_squares_function)
@@ -123,6 +130,13 @@ class Minuit:
 
     @errordef.setter
     def errordef(self, value: float) -> None:
+        fcn_errordef = getattr(self._fcn._fcn, "errordef", None)
+        if fcn_errordef is not None:
+            msg = (
+                f"cost function has an errordef attribute equal to {fcn_errordef}, "
+                "you should not override this with Minuit.errordef"
+            )
+            warnings.warn(msg, ErrordefAlreadySetWarning)
         if value <= 0:
             raise ValueError(f"errordef={value} must be a positive number")
         self._fcn._errordef = value
@@ -488,9 +502,9 @@ class Minuit:
 
     def __init__(
         self,
-        fcn: Callable,
-        *args: Union[float, _ArrayLike[float]],
-        grad: Callable = None,
+        fcn: Cost,
+        *args: Union[float, ArrayLike],
+        grad: Union[CostGradient, bool, None] = None,
         name: Collection[str] = None,
         **kwds: float,
     ):
@@ -508,11 +522,18 @@ class Minuit:
         *args :
             Starting values for the minimization as positional arguments.
             See notes for details on how to set starting values.
-        grad :
-            Function that calculates the gradient and returns an iterable object with
-            one entry for each parameter, which is the derivative for that parameter. If
-            None (default), Minuit will calculate the gradient numerically.
-        name :
+        grad : callable, bool, or None, optional
+            If grad is a callable, it must be a function that calculates the gradient
+            and returns an iterable object with one entry for each parameter, which is
+            the derivative of `fcn` for that parameter. If None (default), Minuit will
+            call the function :func:`iminuit.util.gradient` on `fcn`. If this function
+            returns a callable, it will be used, otherwise Minuit will internally
+            compute the gradient numerically. Please see the documentation of
+            :func:`iminuit.util.gradient` how gradients are detected. Passing a boolean
+            override this detection. If grad=True is used, a ValueError is raised if no
+            useable gradient is found. If grad=False, Minuit will internally compute the
+            gradient numerically.
+        name : sequence of str, optional
             If it is set, it overrides iminuit's function signature detection.
         **kwds :
             Starting values for the minimization as keyword arguments.
@@ -605,7 +626,7 @@ class Minuit:
 
         See Also
         --------
-        migrad, hesse, minos, scan, simplex
+        migrad, hesse, minos, scan, simplex, iminuit.util.gradient
         """
         array_call = False
         if len(args) == 1 and isinstance(args[0], Iterable):
@@ -637,9 +658,24 @@ class Minuit:
         # set self.tol to default value
         self.tol = None  # type:ignore
         self._strategy = MnStrategy(1)
+
+        if grad is None:
+            grad = mutil.gradient(fcn)
+        elif grad is True:
+            grad = mutil.gradient(fcn)
+            if grad is None:
+                raise ValueError(
+                    "gradient enforced, but iminuit.util.gradient returned None"
+                )
+        elif grad is False:
+            grad = None
+
+        if grad is not None and not isinstance(grad, CostGradient):
+            raise ValueError("provided gradient is not a CostGradient")
+
         self._fcn = FCN(
             fcn,
-            getattr(fcn, "grad", grad),
+            grad,
             array_call,
             getattr(fcn, "errordef", 1.0),
         )
@@ -708,7 +744,12 @@ class Minuit:
         self._covariance: mutil.Matrix = None
         return self  # return self for method chaining and to autodisplay current state
 
-    def migrad(self, ncall: int = None, iterate: int = 5) -> "Minuit":
+    def migrad(
+        self,
+        ncall: Optional[int] = None,
+        iterate: int = 5,
+        use_simplex: bool = True,
+    ) -> "Minuit":
         """
         Run Migrad minimization.
 
@@ -720,42 +761,41 @@ class Minuit:
 
         Parameters
         ----------
-        ncall :
+        ncall : int or None, optional
             Approximate maximum number of calls before minimization will be aborted.
             If set to None, use the adaptive heuristic from the Minuit2 library
             (Default: None). Note: The limit may be slightly violated, because the
             condition is checked only after a full iteration of the algorithm, which
             usually performs several function calls.
-
-        iterate :
+        iterate : int, optional
             Automatically call Migrad up to N times if convergence was not reached
             (Default: 5). This simple heuristic makes Migrad converge more often even if
             the numerical precision of the cost function is low. Setting this to 1
             disables the feature.
+        use_simplex: bool, optional
+            If we have to iterate, set this to True to call the Simplex algorithm before
+            each call to Migrad (Default: True). This may improve convergence in
+            pathological cases (which we are in when we have to iterate).
 
         See Also
         --------
         simplex, scan
         """
-        if ncall is None:
-            ncall = 0  # tells C++ Minuit to use its internal heuristic
-
         if iterate < 1:
             raise ValueError("iterate must be at least 1")
 
-        migrad = MnMigrad(self._fcn, self._last_state, self.strategy)
-
         t = mutil._Timer(self._fmin)
         with t:
-            # Automatically call Migrad up to `iterate` times if minimum is not valid.
-            # This simple heuristic makes Migrad converge more often.
-            for _ in range(iterate):
-                # workaround: precision must be set again after each call to MnMigrad
-                if self._precision is not None:
-                    migrad.precision = self._precision
-                fm = migrad(ncall, self._tolerance)
-                if fm.is_valid or fm.has_reached_call_limit:
-                    break
+            fm = _robust_low_level_fit(
+                self._fcn,
+                self._last_state,
+                replace_none(ncall, 0),
+                self._strategy,
+                self._tolerance,
+                self._precision,
+                iterate,
+                use_simplex,
+            )
 
         self._last_state = fm.state
 
@@ -772,7 +812,7 @@ class Minuit:
 
         return self  # return self for method chaining and to autodisplay current state
 
-    def simplex(self, ncall: int = None) -> "Minuit":
+    def simplex(self, ncall: Optional[int] = None) -> "Minuit":
         """
         Run Simplex minimization.
 
@@ -806,16 +846,14 @@ class Minuit:
         minimization. Early stopping can be avoided by setting Minuit.tol to an
         accordingly smaller value, however.
         """
-        if ncall is None:
-            ncall = 0  # tells C++ Minuit to use its internal heuristic
-
         simplex = MnSimplex(self._fcn, self._last_state, self.strategy)
         if self._precision is not None:
             simplex.precision = self._precision
 
         t = mutil._Timer(self._fmin)
         with t:
-            fm = simplex(ncall, self._tolerance)
+            # ncall = 0 tells C++ Minuit to use its internal heuristic
+            fm = simplex(replace_none(ncall, 0), self._tolerance)
         self._last_state = fm.state
 
         self._fmin = mutil.FMin(
@@ -832,7 +870,7 @@ class Minuit:
 
         return self  # return self for method chaining and to autodisplay current state
 
-    def scan(self, ncall: int = None) -> "Minuit":
+    def scan(self, ncall: Optional[int] = None) -> "Minuit":
         """
         Brute-force minimization.
 
@@ -907,7 +945,7 @@ class Minuit:
 
         def run(ipar: int) -> None:
             if ipar == self.npar:
-                r = self.fcn(x[: self.npar])
+                r = self._fcn(x[: self.npar])
                 if r < x[self.npar]:
                     x[self.npar] = r
                     self.values[:] = x[: self.npar]
@@ -945,7 +983,7 @@ class Minuit:
     def scipy(
         self,
         method: Union[str, Callable] = None,
-        ncall: int = None,
+        ncall: Optional[int] = None,
         hess: Any = None,
         hessp: Any = None,
         constraints: Iterable = None,
@@ -1107,9 +1145,9 @@ class Minuit:
                         self.vec[self.free] = v
                         return self.fcn(*self.par, self.vec)[self.free]
 
-        fcn = Wrapped(self.fcn._fcn)
+        fcn = Wrapped(self._fcn._fcn)
 
-        grad = self.fcn._grad
+        grad = self._fcn._grad
         grad = WrappedGrad(grad) if grad else None
 
         if hess:
@@ -1208,9 +1246,11 @@ class Minuit:
                 fcn,
                 start,
                 method=method,
-                bounds=Bounds(lower_bound, upper_bound, keep_feasible=True)
-                if has_limits
-                else None,
+                bounds=(
+                    Bounds(lower_bound, upper_bound, keep_feasible=True)
+                    if has_limits
+                    else None
+                ),
                 jac=grad,
                 hess=hess,
                 hessp=hessp,
@@ -1220,9 +1260,9 @@ class Minuit:
         if self.print_level > 0:
             print(r)
 
-        self.fcn._nfcn += r["nfev"]
+        self._fcn._nfcn += r["nfev"]
         if grad:
-            self.fcn._ngrad += r.get("njev", 0)
+            self._fcn._ngrad += r.get("njev", 0)
 
         # Get inverse Hesse matrix, working around many inconsistencies in scipy.
         # Try in order:
@@ -1240,7 +1280,7 @@ class Minuit:
             matrix = r.hess
             needs_invert = True
         # hess_inv is a function, need to convert to full matrix
-        if hasattr(matrix, "__call__"):
+        if callable(matrix):
             assert matrix is not None  # for mypy
             matrix = matrix(np.eye(self.nfit))
         accurate_covar = bool(hess) or bool(hessp)
@@ -1327,16 +1367,20 @@ class Minuit:
         kwargs :
             Other keyword arguments are forwarded to the
             plot function.
+
+        See Also
+        --------
+        Minuit.interactive
         """
         return self._visualize(plot)(self.values, **kwargs)
 
-    def hesse(self, ncall: int = None) -> "Minuit":
+    def hesse(self, ncall: Optional[int] = None) -> "Minuit":
         """
         Run Hesse algorithm to compute asymptotic errors.
 
         The Hesse method estimates the covariance matrix by inverting the matrix of
         `second derivatives (Hesse matrix) at the minimum
-        <http://en.wikipedia.org/wiki/Hessian_matrix>`_. To get parameters correlations,
+        <https://en.wikipedia.org/wiki/Hessian_matrix>`_. To get parameters correlations,
         you need to use this. The Minos algorithm is another way to estimate parameter
         uncertainties, see :meth:`minos`.
 
@@ -1363,8 +1407,6 @@ class Minuit:
         --------
         minos
         """
-        ncall = 0 if ncall is None else int(ncall)
-
         # Should be fixed upstream: workaround for segfault in MnHesse when all
         # parameters are fixed
         if self.nfit == 0:
@@ -1397,7 +1439,8 @@ class Minuit:
 
         t = mutil._Timer(self._fmin)
         with t:
-            hesse(self._fcn, fm, ncall, self._fmin.edm_goal)
+            # ncall = 0 tells C++ Minuit to use its internal heuristic
+            hesse(self._fcn, fm, replace_none(ncall, 0), self._fmin.edm_goal)
 
         self._last_state = fm.state
         self._fmin = mutil.FMin(
@@ -1418,7 +1461,7 @@ class Minuit:
         self,
         *parameters: Union[int, str],
         cl: float = None,
-        ncall: int = None,
+        ncall: Optional[int] = None,
     ) -> "Minuit":
         """
         Run Minos algorithm to compute confidence intervals.
@@ -1463,8 +1506,6 @@ class Minuit:
         minimisation for all other parameters of the cost function for each scan point.
         This requires many more function evaluations than running the Hesse algorithm.
         """
-        ncall = 0 if ncall is None else int(ncall)
-
         factor = _cl_to_errordef(cl, 1, 1.0)
 
         if self._fmin_does_not_exist_or_last_state_was_modified():
@@ -1496,7 +1537,7 @@ class Minuit:
                 minos = MnMinos(self._fcn, fm, self.strategy)
                 for ipar in ipars:
                     par = self._pos2var[ipar]
-                    me = minos(ipar, ncall, self._tolerance)
+                    me = minos(ipar, replace_none(ncall, 0), self._tolerance)
                     self._merrors[par] = mutil.MError(
                         me.number,
                         par,
@@ -1528,8 +1569,11 @@ class Minuit:
         *,
         size: int = 30,
         bound: Union[float, UserBound] = 2,
-        grid: _ArrayLike[float] = None,
+        grid: ArrayLike = None,
         subtract_min: bool = False,
+        ncall: int = 0,
+        iterate: int = 5,
+        use_simplex: bool = True,
     ) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
         r"""
         Get Minos profile over a specified interval.
@@ -1552,6 +1596,21 @@ class Minuit:
             bound are ignored.
         subtract_min : bool, optional
             If true, subtract offset so that smallest value is zero (Default: False).
+        ncall : int, optional
+            Approximate maximum number of calls before minimization will be aborted.
+            If set to 0, use the adaptive heuristic from the Minuit2 library
+            (Default: 0). Note: The limit may be slightly violated, because the
+            condition is checked only after a full iteration of the algorithm, which
+            usually performs several function calls.
+        iterate : int, optional
+            Automatically call Migrad up to N times if convergence was not reached
+            (Default: 5). This simple heuristic makes Migrad converge more often even if
+            the numerical precision of the cost function is low. Setting this to 1
+            disables the feature.
+        use_simplex: bool, optional
+            If we have to iterate, set this to True to call the Simplex algorithm before
+            each call to Migrad (Default: True). This may improve convergence in
+            pathological cases (which we are in when we have to iterate).
 
         Returns
         -------
@@ -1561,6 +1620,10 @@ class Minuit:
             Profile values.
         array of bool
             Whether minimisation in each point succeeded or not.
+
+        See Also
+        --------
+        profile, contour, mncontour
         """
         ipar, pname = self._normalize_key(vname)
         del vname
@@ -1578,10 +1641,20 @@ class Minuit:
 
         state = MnUserParameterState(self._last_state)  # copy
         state.fix(ipar)
+        # strategy 0 to avoid expensive computation of Hesse matrix
+        strategy = MnStrategy(0)
         for i, v in enumerate(x):
             state.set_value(ipar, v)
-            migrad = MnMigrad(self._fcn, state, self.strategy)
-            fm = migrad(0, self._tolerance)
+            fm = _robust_low_level_fit(
+                self._fcn,
+                state,
+                ncall,
+                strategy,
+                self._tolerance,
+                self._precision,
+                iterate,
+                use_simplex,
+            )
             if not fm.is_valid:
                 warnings.warn(
                     f"MIGRAD fails to converge for {pname}={v}", mutil.IMinuitWarning
@@ -1605,12 +1678,15 @@ class Minuit:
 
         Parameters
         ----------
+        vname: int or string
+            Which variable to scan over, can be identified by index or name.
         band : bool, optional
             If true, show a band to indicate the Hesse error interval (Default: True).
-
         text : bool, optional
             If true, show text a title with the function value and the Hesse error
             (Default: True).
+        **kwargs :
+            Other keyword arguments are forwarded to :meth:`mnprofile`.
 
         Examples
         --------
@@ -1619,7 +1695,7 @@ class Minuit:
 
         See Also
         --------
-        mnprofile, profile, draw_profile
+        mnprofile, draw_profile, draw_contour, draw_mnmatrix
         """
         ipar, pname = self._normalize_key(vname)
         del vname
@@ -1634,7 +1710,7 @@ class Minuit:
         *,
         size: int = 100,
         bound: Union[float, UserBound] = 2,
-        grid: _ArrayLike[float] = None,
+        grid: ArrayLike = None,
         subtract_min: bool = False,
     ) -> Tuple[np.ndarray, np.ndarray]:
         r"""
@@ -1669,7 +1745,7 @@ class Minuit:
 
         See Also
         --------
-        mnprofile
+        mnprofile, contour, mncontour
         """
         ipar, par = self._normalize_key(vname)
         del vname
@@ -1686,7 +1762,7 @@ class Minuit:
         values = np.array(self.values)
         for i, vi in enumerate(x):
             values[ipar] = vi
-            y[i] = self.fcn(values)
+            y[i] = self._fcn(values)
 
         if subtract_min:
             y -= np.min(y)
@@ -1713,7 +1789,7 @@ class Minuit:
 
         See Also
         --------
-        profile, mnprofile, draw_mnprofile
+        profile, draw_mnprofile, draw_contour, draw_mncontourprofile
         """
         ipar, pname = self._normalize_key(vname)
         del vname
@@ -1755,10 +1831,14 @@ class Minuit:
 
         if text:
             plt.title(
-                (f"{pname} = {v:.3g}")
-                if vmin is None
-                else (
-                    "{} = {:.3g} - {:.3g} + {:.3g}".format(pname, v, v - vmin, vmax - v)
+                (
+                    (f"{pname} = {v:.3g}")
+                    if vmin is None
+                    else (
+                        "{} = {:.3g} - {:.3g} + {:.3g}".format(
+                            pname, v, v - vmin, vmax - v
+                        )
+                    )
                 ),
                 fontsize="large",
             )
@@ -1772,7 +1852,7 @@ class Minuit:
         *,
         size: int = 50,
         bound: Union[float, Iterable[Tuple[float, float]]] = 2,
-        grid: Tuple[_ArrayLike, _ArrayLike] = None,
+        grid: Tuple[ArrayLike, ArrayLike] = None,
         subtract_min: bool = False,
     ) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
         r"""
@@ -1819,7 +1899,7 @@ class Minuit:
 
         See Also
         --------
-        mncontour, mnprofile
+        mncontour, mnprofile, profile
         """
         ix, xname = self._normalize_key(x)
         iy, yname = self._normalize_key(y)
@@ -1876,7 +1956,7 @@ class Minuit:
 
         See Also
         --------
-        contour, mncontour, draw_mncontour
+        contour, draw_mncontour, draw_profile, draw_mnmatrix
         """
         from matplotlib import pyplot as plt
 
@@ -1908,6 +1988,9 @@ class Minuit:
         size: int = 100,
         interpolated: int = 0,
         experimental: bool = False,
+        ncall: int = 0,
+        iterate: int = 5,
+        use_simplex: bool = True,
     ) -> np.ndarray:
         """
         Get 2D Minos confidence region.
@@ -1955,6 +2038,21 @@ class Minuit:
             found to succeed in cases where MnContour produced no reasonable result, but
             is slower and not yet well tested in practice. Use with caution and report
             back any issues via Github.
+        ncall : int, optional
+            This parameter only takes effect if ``experimental`` is True.
+            Approximate maximum number of calls before minimization will be aborted. If
+            set to 0, use the adaptive heuristic from the Minuit2 library (Default: 0).
+        iterate : int, optional
+            This parameter only takes effect if ``experimental`` is True.
+            Automatically call Migrad up to N times if convergence was not reached
+            (Default: 5). This simple heuristic makes Migrad converge more often even if
+            the numerical precision of the cost function is low. Setting this to 1
+            disables the feature.
+        use_simplex: bool, optional
+            This parameter only takes effect if ``experimental`` is True.
+            If we have to iterate, set this to True to call the Simplex algorithm before
+            each call to Migrad (Default: True). This may improve convergence in
+            pathological cases (which we are in when we have to iterate).
 
         Returns
         -------
@@ -1965,7 +2063,7 @@ class Minuit:
 
         See Also
         --------
-        contour, mnprofile
+        contour, mnprofile, profile
         """
         ix, xname = self._normalize_key(x)
         iy, yname = self._normalize_key(y)
@@ -1987,7 +2085,9 @@ class Minuit:
             )
 
         if experimental:
-            ce = self._experimental_mncontour(factor, ix, iy, size)
+            ce = self._experimental_mncontour(
+                factor, ix, iy, size, ncall, iterate, use_simplex
+            )
         else:
             with _TemporaryErrordef(self._fcn, factor):
                 assert self._fmin is not None
@@ -2012,7 +2112,7 @@ class Minuit:
         x: Union[int, str],
         y: Union[int, str],
         *,
-        cl: Union[float, _ArrayLike[float]] = None,
+        cl: Union[float, ArrayLike] = None,
         size: int = 100,
         interpolated: int = 0,
         experimental: bool = False,
@@ -2035,19 +2135,20 @@ class Minuit:
 
         See Also
         --------
-        mncontour
+        mncontour, draw_contour, draw_mnmatrix, draw_profile
         """
-        from matplotlib import __version__ as mpl_version
+        from matplotlib import __version__ as mpl_version_string
         from matplotlib import pyplot as plt
         from matplotlib.path import Path
         from matplotlib.contour import ContourSet
+        from ._parse_version import parse_version
 
         ix, xname = self._normalize_key(x)
         iy, yname = self._normalize_key(y)
 
-        mpl_version = tuple(map(int, mpl_version.split(".")))
+        mpl_version = parse_version(mpl_version_string)
 
-        cls = [mutil._replace_none(x, 0.68) for x in mutil._iterate(cl)]
+        cls = [replace_none(x, 0.68) for x in mutil._iterate(cl)]
 
         c_val = []
         c_pts = []
@@ -2078,7 +2179,7 @@ class Minuit:
     def draw_mnmatrix(
         self,
         *,
-        cl: Union[float, _ArrayLike[float]] = None,
+        cl: Union[float, ArrayLike] = None,
         size: int = 100,
         experimental: bool = False,
         figsize=None,
@@ -2119,7 +2220,7 @@ class Minuit:
 
         See Also
         --------
-        mncontour
+        mncontour, mnprofile, draw_mncontour, draw_contour, draw_profile
         """
         if not self.valid:
             raise RuntimeError(f"Function minimum is not valid: {repr(self._fmin)}")
@@ -2130,7 +2231,7 @@ class Minuit:
         if npar == 0:
             raise RuntimeError("all parameters are fixed")
 
-        cls = [mutil._replace_none(x, 0.68) for x in mutil._iterate(cl)]
+        cls = [replace_none(x, 0.68) for x in mutil._iterate(cl)]
         if len(cls) == 0:
             raise ValueError("cl must have at least one value")
 
@@ -2173,8 +2274,9 @@ class Minuit:
                         extremes.append(x[k - 1])
                     if yk > fmax and y[k - 1] < fmax:
                         extremes.append(xk)
-                a = min(*extremes, a)
-                b = max(*extremes, b)
+                if extremes:
+                    a = min(*extremes, a)
+                    b = max(*extremes, b)
                 prange[par1] = (a, b)
                 plt.ylim(0, fmax + 0.5)
                 for j in range(i):
@@ -2240,211 +2342,15 @@ class Minuit:
         --------
         .. plot:: plots/interactive.py
             :include-source:
+
+        See Also
+        --------
+        Minuit.visualize
         """
-        with warnings.catch_warnings():
-            # ipywidgets produces deprecation warnings through use of internal APIs :(
-            warnings.simplefilter("ignore")
-            try:
-                from ipywidgets import (
-                    HBox,
-                    VBox,
-                    Output,
-                    FloatSlider,
-                    Button,
-                    ToggleButton,
-                    Layout,
-                    Dropdown,
-                )
-                from ipywidgets.widgets.interaction import show_inline_matplotlib_plots
-                from IPython.display import clear_output
-                from matplotlib import pyplot as plt
-            except ModuleNotFoundError as e:
-                e.msg += (
-                    "\n\nPlease install ipywidgets, IPython, and matplotlib to "
-                    "enable interactive"
-                )
-                raise
+        from iminuit.ipywidget import make_widget
 
-            plot = self._visualize(plot)
-
-            def plot_with_frame(args, from_fit, report_success):
-                trans = plt.gca().transAxes
-                try:
-                    with warnings.catch_warnings():
-                        if self.fcn._array_call:
-                            plot([args], **kwargs)  # prevent unpacking of array
-                        else:
-                            plot(args, **kwargs)
-                except Exception:
-                    if raise_on_exception:
-                        raise
-
-                    import traceback
-
-                    plt.figtext(
-                        0.01,
-                        0.5,
-                        traceback.format_exc(),
-                        ha="left",
-                        va="center",
-                        transform=trans,
-                        color="r",
-                    )
-                    return
-                if from_fit:
-                    fval = self.fmin.fval
-                else:
-                    fval = self.fcn(args)
-                plt.text(
-                    0.05,
-                    1.05,
-                    f"FCN = {fval:.3f}",
-                    transform=trans,
-                    fontsize="x-large",
-                )
-                if from_fit and report_success:
-                    plt.text(
-                        0.95,
-                        1.05,
-                        f"{'success' if self.valid and self.accurate else 'FAILURE'}",
-                        transform=trans,
-                        fontsize="x-large",
-                        ha="right",
-                    )
-
-            class ParameterBox(HBox):
-                def __init__(self, par, val, min, max, step, fix):
-                    self.par = par
-                    self.slider = FloatSlider(
-                        val,
-                        min=a,
-                        max=b,
-                        step=step,
-                        description=par,
-                        continuous_update=True,
-                        layout=Layout(min_width="70%"),
-                    )
-                    self.fix = ToggleButton(
-                        fix, description="Fix", layout=Layout(width="3.1em")
-                    )
-                    self.opt = ToggleButton(
-                        False, description="Opt", layout=Layout(width="3.5em")
-                    )
-                    self.opt.observe(self.on_opt_toggled, "value")
-                    super().__init__([self.slider, self.fix, self.opt])
-
-                def on_opt_toggled(self, change):
-                    self.slider.disabled = self.opt.value
-                    on_slider_change(None)
-
-            def fit():
-                if algo_choice.value == "Migrad":
-                    self.migrad()
-                elif algo_choice.value == "Scipy":
-                    self.scipy()
-                elif algo_choice.value == "Simplex":
-                    self.simplex()
-                    return False
-                else:
-                    assert False  # pragma: no cover, should never happen
-                return True
-
-            def on_slider_change(change):
-                if out.block:
-                    return
-                args = [x.slider.value for x in parameters]
-                from_fit = False
-                report_success = False
-                if any(x.opt.value for x in parameters):
-                    save = self.fixed[:]
-                    self.fixed = [not x.opt.value for x in parameters]
-                    self.values = args
-                    report_success = fit()
-                    args = self.values[:]
-                    out.block = True
-                    for x, val in zip(parameters, args):
-                        x.slider.value = val
-                    out.block = False
-                    self.fixed = save
-                    from_fit = True
-                with out:
-                    clear_output(wait=True)
-                    plot_with_frame(args, from_fit, report_success)
-                    with warnings.catch_warnings():
-                        warnings.simplefilter("ignore")
-                        show_inline_matplotlib_plots()
-
-            def on_fit_button_clicked(change):
-                for x in parameters:
-                    self.values[x.par] = x.slider.value
-                    self.fixed[x.par] = x.fix.value
-                report_success = fit()
-                out.block = True
-                for x in parameters:
-                    val = self.values[x.par]
-                    if val < x.slider.min:
-                        x.slider.min = val
-                    elif val > x.slider.max:
-                        x.slider.max = val
-                    x.slider.value = val
-                out.block = False
-                with out:
-                    clear_output(wait=True)
-                    plot_with_frame(self.values, True, report_success)
-                    show_inline_matplotlib_plots()
-
-            def on_update_button_clicked(change):
-                for x in parameters:
-                    x.slider.continuous_update = not x.slider.continuous_update
-
-            def on_reset_button_clicked(change):
-                self.reset()
-                out.block = True
-                for x in parameters:
-                    x.slider.value = self.values[x.par]
-                out.block = False
-                on_slider_change(None)
-
-            parameters = []
-            for par in self.parameters:
-                val = self.values[par]
-                step = mutil._guess_initial_step(val)
-                a, b = self.limits[par]
-                # safety margin to avoid overflow warnings
-                a = a + 1e-300 if np.isfinite(a) else val - 100 * step
-                b = b - 1e-300 if np.isfinite(b) else val + 100 * step
-                parameters.append(ParameterBox(par, val, a, b, step, self.fixed[par]))
-
-            fit_button = Button(description="Fit")
-            fit_button.on_click(on_fit_button_clicked)
-
-            update_button = ToggleButton(True, description="Continuous")
-            update_button.observe(on_update_button_clicked)
-
-            reset_button = Button(description="Reset")
-            reset_button.on_click(on_reset_button_clicked)
-
-            algo_choice = Dropdown(
-                options=["Migrad", "Scipy", "Simplex"], value="Migrad"
-            )
-
-            ui = VBox(
-                [
-                    HBox([fit_button, update_button, reset_button, algo_choice]),
-                    VBox(parameters),
-                ]
-            )
-
-            out = Output()
-            out.block = False
-
-            for x in parameters:
-                x.slider.observe(on_slider_change, "value")
-
-            show_inline_matplotlib_plots()
-            on_slider_change(None)
-
-            return HBox([out, ui])
+        plot = self._visualize(plot)
+        return make_widget(self, plot, kwargs, raise_on_exception)
 
     def _free_parameters(self) -> Set[str]:
         return set(mp.name for mp in self._last_state if not mp.is_fixed)
@@ -2523,7 +2429,8 @@ class Minuit:
         # - goal is used to detect convergence but violations by 10x are also accepted;
         #   see VariableMetricBuilder.cxx:425
         edm_goal = max(
-            self.tol * self.errordef, self._mnprecision().eps2  # type:ignore
+            self.tol * self.errordef,
+            self._mnprecision().eps2,  # type:ignore
         )
         if migrad_factor:
             edm_goal *= 2e-3
@@ -2591,19 +2498,27 @@ class Minuit:
             p.text(str(self))
 
     def _visualize(self, plot):
-        pyfcn = self.fcn._fcn
+        pyfcn = self._fcn._fcn
         if plot is None:
             if hasattr(pyfcn, "visualize"):
                 plot = pyfcn.visualize
             else:
-                raise AttributeError(
+                msg = (
                     f"class {pyfcn.__class__.__name__} has no visualize method, "
                     "please use the 'plot' keyword to pass a visualization function"
                 )
+                raise AttributeError(msg)
         return plot
 
     def _experimental_mncontour(
-        self, factor: float, ix: int, iy: int, size: int
+        self,
+        factor: float,
+        ix: int,
+        iy: int,
+        size: int,
+        ncall: int,
+        iterate: int,
+        use_simplex: bool,
     ) -> List[Tuple[float, float]]:
         from scipy.optimize import root_scalar
 
@@ -2617,6 +2532,9 @@ class Minuit:
         )
         s = (t * factor) ** 0.5
 
+        # strategy 0 to avoid expensive computation of Hesse matrix
+        strategy = MnStrategy(0)
+
         ce = []
         for phi in np.linspace(-np.pi, np.pi, size, endpoint=False):
 
@@ -2641,8 +2559,16 @@ class Minuit:
                 xy = args(z)
                 state.set_value(ix, xy[0])
                 state.set_value(iy, xy[1])
-                migrad = MnMigrad(self._fcn, state, max(0, self.strategy.strategy - 1))
-                fm = migrad(0, self._tolerance)
+                fm = _robust_low_level_fit(
+                    self._fcn,
+                    state,
+                    ncall,
+                    strategy,
+                    self._tolerance,
+                    self._precision,
+                    iterate,
+                    use_simplex,
+                )
                 return fm.fval - self.fval - factor * self._fcn._errordef
 
             # find bracket
@@ -2798,3 +2724,40 @@ def _cl_to_errordef(cl, npar, default):
         factor = chi2(npar).ppf(cl)  # convert confidence level to errordef
 
     return factor
+
+
+def _robust_low_level_fit(
+    fcn: FCN,
+    state: MnUserParameterState,
+    ncall: int,
+    strategy: MnStrategy,
+    tolerance: float,
+    precision: Optional[float],
+    iterate: int,
+    use_simplex: bool,
+) -> FunctionMinimum:
+    # Automatically call Migrad up to `iterate` times if minimum is not valid.
+    # This simple heuristic makes Migrad converge more often. Optionally,
+    # one can interleave calls to Simplex and Migrad, which may also help.
+    migrad = MnMigrad(fcn, state, strategy)
+    if precision is not None:
+        migrad.precision = precision
+    fm = migrad(ncall, tolerance)
+    strategy = MnStrategy(2)
+    migrad = MnMigrad(fcn, fm.state, strategy)
+    while not fm.is_valid and not fm.has_reached_call_limit and iterate > 1:
+        # If we have to iterate, we have a pathological case. Increasing the
+        # strategy to 2 in this case was found to be beneficial.
+        if use_simplex:
+            simplex = MnSimplex(fcn, fm.state, strategy)
+            if precision is not None:
+                simplex.precision = precision
+            fm = simplex(ncall, tolerance)
+            # recreate MnMigrad instance to start from updated state
+            migrad = MnMigrad(fcn, fm.state, strategy)
+        # workaround: precision must be set again after each call
+        if precision is not None:
+            migrad.precision = precision
+        fm = migrad(ncall, tolerance)
+        iterate -= 1
+    return fm
