@@ -193,7 +193,7 @@ def test_Func1():
 
 
 def test_Func2():
-    with pytest.warns(np.VisibleDeprecationWarning):
+    with pytest.warns(FutureWarning):
         func_test_helper(Func2())
 
 
@@ -1356,7 +1356,7 @@ def test_parameter_at_limit(sign):
 @pytest.mark.parametrize("iterate,valid", ((1, False), (5, True)))
 def test_inaccurate_fcn(iterate, valid):
     def f(x):
-        return abs(x) ** 10 + 1e7
+        return abs(x) ** 10 + 1e6
 
     m = Minuit(f, x=2)
     m.migrad(iterate=iterate)
@@ -1727,3 +1727,32 @@ def test_errordef_already_set_warning():
     # check that cost.errordef value is still overridden
     m.hesse()
     assert_allclose(m.errors, [2, 2])
+
+
+def test_mnprofile_bad_cost():
+    def fn(a, b):
+        if b > 0:
+            return a**2
+        return (a - 0.1) ** 2
+
+    m = Minuit(fn, 1, 2)
+    # test iterative fitting with custom precision
+    # m.precision = 1e-18
+    m.migrad()
+    with pytest.warns(IMinuitWarning, match="MIGRAD fails to converge"):
+        m.mnprofile("a")
+
+
+def test_migrad_iterative_with_precision():
+    def fn(a, b):
+        return 0
+
+    m1 = Minuit(fn, 1, 2)
+    m1.precision = 1e-7
+    m1.migrad(iterate=5)
+
+    m2 = Minuit(fn, 1, 2)
+    m2.precision = 1e-7
+    m2.migrad(iterate=1)
+
+    assert m2.fmin.nfcn < m1.fmin.nfcn
