@@ -1,11 +1,12 @@
-# flake8: noqa E501
+# flake8: noqa: E501
 from iminuit import Minuit
-from iminuit.util import Params, Param, Matrix, FMin, MError
+from iminuit.util import Param, Matrix, FMin, MError
 from iminuit import _repr_html, _repr_text
 import pytest
 from argparse import Namespace
 from pathlib import Path
 import numpy as np
+from iminuit._hide_modules import hide_modules
 
 nan = float("nan")
 inf = float("infinity")
@@ -80,23 +81,23 @@ def test_pdg_format():
 
 def test_matrix_format():
     def a(*args):
-        return np.array(args)
+        return np.reshape(args, (2, 2))
 
     assert _repr_text.matrix_format(a(1e1, 2e2, -3e3, -4e4)) == [
         "10",
         "200",
-        "-3e+03",
+        "-3.00e3",
         "-4e+04",
     ]
-    assert _repr_text.matrix_format(a(nan, 2e2, -nan, -4e4)) == [
-        "nan",
+    assert _repr_text.matrix_format(a(2e2, nan, -nan, -4e4)) == [
         "200",
         "nan",
+        "nan",
         "-4e+04",
     ]
-    assert _repr_text.matrix_format(a(inf, 2e2, -nan, -4e4)) == [
+    assert _repr_text.matrix_format(a(2e2, inf, -nan, -4e4)) == [
+        "200",
         "inf",
-        "200",
         "nan",
         "-4e+04",
     ]
@@ -167,6 +168,28 @@ def fmin_bad():
     return FMin(fm, "SciPy[L-BFGS-B]", 100000, 200000, 1, 1e-5, 1.2)
 
 
+@pytest.fixture
+def fmin_no_cov():
+    fm = Namespace(
+        fval=11.456,
+        edm=1.23456e-10,
+        up=0.5,
+        is_valid=True,
+        has_valid_parameters=True,
+        has_accurate_covar=False,
+        has_posdef_covar=False,
+        has_made_posdef_covar=False,
+        hesse_failed=False,
+        has_covariance=False,
+        is_above_max_edm=False,
+        has_reached_call_limit=False,
+        has_parameters_at_limit=False,
+        errordef=1,
+        state=[],
+    )
+    return FMin(fm, "Migrad", 10, 3, 10, 1e-4, 0.01)
+
+
 def test_html_fmin_good(fmin_good):
     assert fmin_good._repr_html_() == ref("fmin_good.html").format(
         good=_repr_html.good_style
@@ -179,6 +202,12 @@ def test_html_fmin_bad(fmin_bad):
     )
 
 
+def test_html_fmin_no_cov(fmin_no_cov):
+    assert fmin_no_cov._repr_html_() == ref("fmin_no_cov.html").format(
+        good=_repr_html.good_style, bad=_repr_html.bad_style, warn=_repr_html.warn_style
+    )
+
+
 def test_html_params(minuit):
     assert minuit.init_params._repr_html_() == ref("params_init.html")
     assert minuit.params._repr_html_() == ref("params.html")
@@ -240,10 +269,96 @@ def test_text_fmin_bad(fmin_bad):
     assert _repr_text.fmin(fmin_bad) == ref("fmin_bad.txt")
 
 
+def test_text_fmin_no_cov(fmin_no_cov):
+    assert _repr_text.fmin(fmin_no_cov) == ref("fmin_no_cov.txt")
+
+
+def test_text_fmin_not_posdef():
+    fm = Namespace(
+        fval=11.456,
+        edm=1.23456e-10,
+        up=0.5,
+        is_valid=True,
+        has_valid_parameters=True,
+        has_accurate_covar=False,
+        has_posdef_covar=False,
+        has_made_posdef_covar=False,
+        hesse_failed=True,
+        has_covariance=False,
+        is_above_max_edm=False,
+        has_reached_call_limit=False,
+        has_parameters_at_limit=False,
+        errordef=1,
+        state=[],
+    )
+    fmin = FMin(fm, "Migrad", 10, 3, 10, 1e-4, 0.01)
+    assert _repr_text.fmin(fmin) == ref("fmin_not_posdef.txt")
+
+
+def test_text_fmin_made_posdef():
+    fm = Namespace(
+        fval=11.456,
+        edm=1.23456e-10,
+        up=0.5,
+        is_valid=True,
+        has_valid_parameters=True,
+        has_accurate_covar=False,
+        has_posdef_covar=True,
+        has_made_posdef_covar=True,
+        hesse_failed=False,
+        has_covariance=True,
+        is_above_max_edm=False,
+        has_reached_call_limit=False,
+        has_parameters_at_limit=False,
+        errordef=1,
+        state=[],
+    )
+    fmin = FMin(fm, "Migrad", 10, 3, 10, 1e-4, 0.01)
+    assert _repr_text.fmin(fmin) == ref("fmin_made_posdef.txt")
+
+
+def test_text_fmin_approximate():
+    fm = Namespace(
+        fval=11.456,
+        edm=1.23456e-10,
+        up=0.5,
+        is_valid=True,
+        has_valid_parameters=True,
+        has_accurate_covar=False,
+        has_posdef_covar=True,
+        has_made_posdef_covar=False,
+        hesse_failed=False,
+        has_covariance=True,
+        is_above_max_edm=False,
+        has_reached_call_limit=False,
+        has_parameters_at_limit=False,
+        errordef=1,
+        state=[],
+    )
+    fmin = FMin(fm, "Migrad", 10, 3, 10, 1e-4, 0.01)
+    assert _repr_text.fmin(fmin) == ref("fmin_approximate.txt")
+
+
 def test_text_params(minuit):
     assert _repr_text.params(minuit.params) == ref("params.txt")
 
 
+def test_text_params_with_latex_names():
+    pytest.importorskip("unicodeitplus")
+
+    m = Minuit(lambda x: x**2, 1, name=[r"$\alpha$"])
+    assert _repr_text.params(m.params) == ref("params_latex_1.txt")
+
+    with hide_modules("unicodeitplus"):
+        from iminuit.warnings import OptionalDependencyWarning
+
+        with pytest.warns(
+            OptionalDependencyWarning,
+            match="rendering simple LaTeX requires optional package 'unicodeitplus'",
+        ):
+            assert _repr_text.params(m.params) == ref("params_latex_2.txt")
+
+
 def test_text_params_with_long_names():
     mps = [
         Param(
