@@ -13,10 +13,10 @@ def npa(*args, **kwargs):
 @pytest.mark.parametrize(
     "func,args",
     (
-        ("_safe_log", (npa(1.2, 0),)),
+        ("log_or_zero", (npa(1.2, 0),)),
         ("_z_squared", (npa(1.2, 0), npa(1.2, 1.0), npa(1.2, 1))),
         ("_unbinned_nll", (npa(1.2, 0),)),
-        ("multinominal_chi2", (npa(1, 0), npa(1.2, 0))),
+        ("multinomial_chi2", (npa(1, 0), npa(1.2, 0))),
         ("chi2", (npa(1.2, 0), npa(1.2, 1.0), npa(1.2, 0.1))),
         ("poisson_chi2", (npa(1, 0), npa(1.2, 0.1))),
         ("_soft_l1_cost", (npa(1.2, 0), npa(1.2, 0.1), npa(1.0, 1.0))),
